#!/usr/bin/perl
###############################################################################
#   Copyright (C) 2001,2005 Craig B. Agricola (craig@theagricolas.org)
#
# See super_tee.c, which should be found in the same package as this
# file for license and distribution.  Do not redistribute this script without
# the entire tar-ball.  The original tar-ball is most likely available from
# somewhere under http://theagricolas.org/, currently
# http://theagricolas.org/craig/
#
###############################################################################

use POSIX qw(:sys_wait_h);

my ($append) = 0;
my ($outfile) = shift(@ARGV);
if ($outfile eq "-a") { $append = 1; $outfile = shift(@ARGV); }
if ((!defined($outfile))||(!@ARGV)) {
  my ($name) = ($0 =~ m#([^/]*)$#)[0];
  print STDERR "Usage: $name <outputfile> <executable> <args...>\n";
  exit(-1);
}

#if (-e $outfile) {
#  print STDERR "$outfile already exists, and I don't want to overwrite it\n";
#  exit(-1);
#}

pipe(TEEOUT_READ, TEEOUT_WRITE);
pipe(TEEERR_READ, TEEERR_WRITE);

if (!($simv_pid=fork())) {
  close(TEEOUT_READ); close(TEEERR_READ);
  open(STDOUT, ">&TEEOUT_WRITE");
  open(STDERR, ">&TEEERR_WRITE");
  exec(@ARGV);
  exit(-1);
} else {
  close(TEEOUT_WRITE); close(TEEERR_WRITE);
  if ($append) { open(OUT, ">>$outfile"); } else { open(OUT, ">$outfile"); }
  select((select(OUT), $|=1)[0]);
  select((select(STDOUT), $|=1)[0]);
  select((select(STDERR), $|=1)[0]);
  my ($rin, $rout, $eout);
  my ($outbuf, $errbuf);

  sub dump_pipes {
    my ($final) = @_;
    my ($buf);
    if (sysread(TEEOUT_READ, $buf, 1024)) { $outbuf .= $buf; }
    if (sysread(TEEERR_READ, $buf, 1024)) { $errbuf .= $buf; }
    while($outbuf=~m#^([^\n]*)\n(.*)$#s) {
      $outbuf=$2; print OUT "STDOUT: $1\n"; print STDOUT "$1\n";
    }
    while($errbuf=~m#^([^\n]*)\n(.*)$#s) {
      $errbuf=$2; print OUT "STDERR: $1\n"; print STDERR "$1\n";
    }
    if ($final) {
      if ($outbuf ne "") { print OUT "STDOUT: $outbuf (missing newline)\n"; }
      if ($errbuf ne "") { print OUT "STDERR: $errbuf (missing newline)\n"; }
      print STDOUT "$outbuf";
      print STDERR "$errbuf";
    }
  }

  $rin = '';
  vec($rin, fileno(TEEOUT_READ), 1) = 1;
  vec($rin, fileno(TEEERR_READ), 1) = 1;
  while(!waitpid($simv_pid, &WNOHANG)) {
    if (select($rout=$rin, undef, $eout=$rin, 0.1)) { dump_pipes(0); }
  }
  dump_pipes(1);
}
